import StringIO

from django.test import TestCase
from docx import Document

from functions import create_language, get_resource_path, create_appointment, create_user, create_subject, \
    create_visit
from web.models import MailTemplate
from web.models.constants import MAIL_TEMPLATE_CONTEXT_APPOINTMENT, MAIL_TEMPLATE_CONTEXT_VISIT, \
    MAIL_TEMPLATE_CONTEXT_SUBJECT
from web.models.mail_template import DATE_FORMAT_SHORT


class MailTemplateModelTests(TestCase):
    def setUp(self):
        self.french_language = create_language("French", "fr_FR")
        self.english_language = create_language("English", "en_GB")
        self.template_file = get_resource_path('upcoming_appointment_FR.docx')
        self.user = create_user()

    def test_get_appointment_mail_templates(self):
        context = MAIL_TEMPLATE_CONTEXT_APPOINTMENT
        function_to_test = MailTemplate.get_appointment_mail_templates
        self.check_get_mail_templates(context, function_to_test)

    def test_get_visit_mail_templates(self):
        context = MAIL_TEMPLATE_CONTEXT_VISIT
        function_to_test = MailTemplate.get_visit_mail_templates
        self.check_get_mail_templates(context, function_to_test)

    def test_get_subject_mail_templates(self):
        context = MAIL_TEMPLATE_CONTEXT_SUBJECT
        function_to_test = MailTemplate.get_subject_mail_templates
        self.check_get_mail_templates(context, function_to_test)

    def check_get_mail_templates(self, context, function_to_test):
        # create french template
        template_name_french = "test_fr"
        appointment_template_french = MailTemplate(name=template_name_french, language=self.french_language,
                                                   context=context,
                                                   template_file=self.template_file)
        appointment_template_french.save()
        # create english template
        template_name_english = "test_en"
        appointment_template_english = MailTemplate(name=template_name_english, language=self.english_language,
                                                    context=context,
                                                    template_file=self.template_file)
        appointment_template_english.save()
        active_templates, inactive_templates = function_to_test([self.french_language])
        self.assertEqual(1, len(active_templates), "only on active template should be returned (for french)")
        self.assertEqual(1, len(inactive_templates), "only on inactive template should be returned (for english)")
        self.assertEqual(template_name_french, active_templates[0].name)
        self.assertEqual(template_name_english, inactive_templates[0].name)
        # now tries with multiple languages
        active_templates, inactive_templates = function_to_test(
            [self.english_language, self.french_language])
        self.assertEqual(2, len(active_templates), "two active templates should be returned (for french and english)")
        self.assertEqual(0, len(inactive_templates), "no inactive template should be returned")
        # also checks the correct sorting
        self.assertEqual(template_name_english, active_templates[0].name)
        self.assertEqual(template_name_french, active_templates[1].name)

        # tries with no languages
        active_templates, inactive_templates = function_to_test([])
        self.assertEqual(0, len(active_templates), "no active templates should be returned")
        self.assertEqual(2, len(inactive_templates), "two inactive templates should be returned (french and english)")

    def test_apply_appointment(self):
        template_name_french = "test_fr"
        appointment = create_appointment()
        appointment_template_french = MailTemplate(name=template_name_french, language=self.french_language,
                                                   context=MAIL_TEMPLATE_CONTEXT_APPOINTMENT,
                                                   template_file=self.template_file)
        stream = StringIO.StringIO()
        appointment_template_french.apply(appointment, self.user, stream)
        doc = Document(stream)
        worker_name = str(self.user.worker)
        location = appointment.location.name
        self.check_doc_contains(doc, [worker_name, location])

    def test_apply_subject(self):
        template_name_french = "test_fr"
        subject = create_subject()
        subject_template_french = MailTemplate(name=template_name_french, language=self.french_language,
                                               context=MAIL_TEMPLATE_CONTEXT_SUBJECT,
                                               template_file=self.template_file)
        stream = StringIO.StringIO()
        subject_template_french.apply(subject, self.user, stream)
        doc = Document(stream)
        worker_name = str(self.user.worker)

        self.check_doc_contains(doc, [worker_name, str(subject), subject.country, subject.nd_number,
                                      subject.get_type_display()])

    def test_apply_visit(self):
        template_name_french = "test_fr"
        visit = create_visit()
        visit_template_french = MailTemplate(name=template_name_french, language=self.french_language,
                                             context=MAIL_TEMPLATE_CONTEXT_VISIT,
                                             template_file=self.template_file)
        stream = StringIO.StringIO()
        visit_template_french.apply(visit, self.user, stream)
        doc = Document(stream)
        worker_name = str(self.user.worker)

        self.check_doc_contains(doc, [worker_name, str(visit.subject), visit.subject.country, visit.subject.nd_number,
                                      visit.subject.get_type_display(),
                                      visit.datetime_begin.strftime(DATE_FORMAT_SHORT),
                                      visit.datetime_end.strftime(DATE_FORMAT_SHORT)])

    def check_doc_contains(self, doc, needles):
        founds = [False] * len(needles)
        all_founds = False
        count_found = 0
        for paragraph in doc.paragraphs:
            for i in range(0, len(needles)):
                if not founds[i] and needles[i] in paragraph.text:
                    founds[i] = True
                    count_found += 1
            if count_found == len(needles):
                all_founds = True
                break
        if not all_founds:
            for i in range(0, len(needles)):
                self.assertTrue(founds[i], "{} was not found in the generated Word document".format(needles[i]))
